BTreeMap

Struct BTreeMap 

pub struct BTreeMap<K: Ord + Clone + Sized, V: Sized> { /* private fields */ }
Available on crate feature btree only.
B+Tree Map for single-threaded usage, optimized for numeric type.

Implementations

Source

impl<K: Ord + Sized + Clone, V: Sized> BTreeMap<K, V>

pub fn new() -> Self

Create a new empty BTreeMap

pub fn len(&self) -> usize

Returns the number of elements in the map

pub fn is_empty(&self) -> bool

Returns true if the map is empty

pub const fn cap() -> (u32, u32)

return (cap_of_inter_node, cap_of_leaf_node)

pub fn leaf_count(&self) -> usize

Return the number of leaf nodes

pub fn get_fill_ratio(&self) -> f32

Return the average fill ratio of leaf nodes

The range is [0.0, 100]

pub fn height(&self) -> u32

When root is leaf, returns 1, otherwise return the number of layers of inter node

pub fn entry(&mut self, key: K) -> Entry<'_, K, V>

Returns an entry to the key in the map

pub fn contains_key<Q>(&self, key: &Q) -> bool
where K: Borrow<Q>, Q: Ord + ?Sized,

Returns true if the map contains the key

pub fn get<Q>(&self, key: &Q) -> Option<&V>
where K: Borrow<Q>, Q: Ord + ?Sized,

Returns a reference to the value corresponding to the key

pub fn get_mut<Q>(&mut self, key: &Q) -> Option<&mut V>
where K: Borrow<Q>, Q: Ord + ?Sized,

Returns a mutable reference to the value corresponding to the key

pub fn insert(&mut self, key: K, value: V) -> Option<V>

Insert a key-value pair into the map Returns the old value if the key already existed

pub fn remove<Q>(&mut self, key: &Q) -> Option<V>
where K: Borrow<Q>, Q: Ord + ?Sized,

Remove a key from the map, returning the value if it existed

pub fn remove_range<R>(&mut self, range: R) -> Option<(K, V)>
where R: RangeBounds<K>,

Perform removal in batch mode and return the last item removed

NOTE: this function speed up by skipping the underflow of LeafNode until the last operation.

pub fn remove_range_with<R, F>(&mut self, range: R, cb: F) -> Option<(K, V)>
where R: RangeBounds<K>, F: FnMut(&K, &V),

Perform removal in batch mode and return the last item removed

On each removal, callback function cb is invoke with (ref of key, ref of value)

NOTE: this function speed up by skipping the underflow of LeafNode until the last operation.

pub fn validate(&self)
where K: Debug, V: Debug,

Validate the entire tree structure Uses the same traversal logic as Drop to avoid recursion

pub fn first_key_value(&self) -> Option<(&K, &V)>

Returns the first key-value pair in the map Returns None if the map is empty

pub fn last_key_value(&self) -> Option<(&K, &V)>

Returns the last key-value pair in the map Returns None if the map is empty

pub fn first_entry(&mut self) -> Option<OccupiedEntry<'_, K, V>>

Returns an entry to the first key in the map Returns None if the map is empty

pub fn last_entry(&mut self) -> Option<OccupiedEntry<'_, K, V>>

Returns an entry to the last key in the map Returns None if the map is empty

pub fn pop_first(&mut self) -> Option<(K, V)>

Removes and returns the first key-value pair in the map Returns None if the map is empty

pub fn pop_last(&mut self) -> Option<(K, V)>

Removes and returns the last key-value pair in the map Returns None if the map is empty

pub fn iter(&self) -> Iter<'_, K, V>

Returns an iterator over the map’s entries

pub fn iter_mut(&mut self) -> IterMut<'_, K, V>

Returns a mutable iterator over the map’s entries

pub fn into_iter_rev(self) -> IntoIter<K, V>

Return a consuming iterator in reversed order

pub fn keys(&self) -> Keys<'_, K, V>

Returns an iterator over the map’s keys

pub fn values(&self) -> Values<'_, K, V>

Returns an iterator over the map’s values

pub fn values_mut(&mut self) -> ValuesMut<'_, K, V>

Returns a mutable iterator over the map’s values

pub fn range<R>(&self, range: R) -> Range<'_, K, V>
where R: RangeBounds<K>,

Returns an iterator over a sub-range of entries in the map

pub fn range_mut<R>(&mut self, range: R) -> RangeMut<'_, K, V>
where R: RangeBounds<K>,

Returns a mutable iterator over a sub-range of entries in the map