OccupiedEntry

Struct OccupiedEntry 

pub struct OccupiedEntry<'a, K: Ord + Clone + Sized, V: Sized> { /* private fields */ }
Available on crate feature btree only.
Entry for an existing key-value pair in the tree

Implementations

Source

impl<'a, K: Ord + Clone + Sized, V: Sized> OccupiedEntry<'a, K, V>

pub fn key(&self) -> &K

Get a reference to the key

pub fn remove(self) -> V

Remove the key-value pair from the tree and return the value

pub fn remove_entry(self) -> (K, V)

Remove the key-value pair from the tree and return the key and value

pub fn get(&self) -> &V

Get a reference to the value

pub fn get_mut(&mut self) -> &mut V

Get a mutable reference to the value

pub fn into_mut(self) -> &'a mut V

Convert the OccupiedEntry into a mutable reference bounded by the tree’s lifetime

pub fn insert(&mut self, value: V) -> V

replace a value into the tree and return the old value

pub fn peak_backward(&self) -> Option<(&'a K, &'a V)>

Peak previous OccupiedEntry

pub fn peak_forward(&self) -> Option<(&'a K, &'a V)>

Peak the next OccupiedEntry

pub fn move_backward(self) -> Result<Self, Self>

Move to previous OccupiedEntry

When reaching the front, return the original entry in Err()

pub fn move_forward(self) -> Result<Self, Self>

Move to next OccupiedEntry

When reaching the end, return the original entry in Err()

pub fn alter_key(&mut self, k: K) -> Result<(), ()>

Try to alter the key of this entry

On successful returns Ok() ; If key is not in strict order among the neighbors, return Err() .