VacantEntry

Struct VacantEntry 

pub struct VacantEntry<'a, K: Ord + Clone + Sized, V: Sized> { /* private fields */ }
Available on crate feature btree only.
Entry for a vacant key position in the tree

Implementations

Source

impl<'a, K: Ord + Clone + Sized, V: Sized> VacantEntry<'a, K, V>

pub fn key(&self) -> &K

Get a reference to the key

pub fn into_key(self) -> K

Take ownership of the key

pub fn insert(self, value: V) -> &'a mut V

Insert a value into the tree

pub fn peak_backward(&self) -> Option<(&'a K, &'a V)>

Peak previous OccupiedEntry

pub fn peak_forward(&self) -> Option<(&'a K, &'a V)>

Peak the next OccupiedEntry

pub fn move_backward(self) -> Result<OccupiedEntry<'a, K, V>, Self>

Move to previous OccupiedEntry

When reaching the front return the original entry in Err()

pub fn move_forward(self) -> Result<OccupiedEntry<'a, K, V>, Self>

Move to next OccupiedEntry

When reaching the end, return the original entry in Err()