VariousMap

Enum VariousMap 

pub enum VariousMap<K, V> {
    One(Option<(K, V)>),
    Multi(BTreeMap<K, V>),
}
A tempoary map optimise for empty or one condition, to delay allocation.

Initial to be Option<(K, V)>. If multi item inserted, transit from Option to std::collections::BTreeMap.

Variants



One(Option<(K, V)>)



Multi(BTreeMap<K, V>)

Implementations

Source

impl<K: Ord, V> VariousMap<K, V>

pub fn new() -> Self

pub fn get<Q>(&self, key: &Q) -> Option<&V>
where K: Borrow<Q> + Ord, Q: Ord + ?Sized,

pub fn get_mut<Q>(&mut self, key: &Q) -> Option<&mut V>
where K: Borrow<Q> + Ord, Q: Ord + ?Sized,

pub fn insert(&mut self, key: K, value: V) -> Option<V>
where K: Ord,

pub fn iter(&self) -> Iter<'_, K, V>

pub fn entry(&mut self, key: K) -> Entry<'_, K, V>

pub fn len(&self) -> usize

pub fn is_empty(&self) -> bool

pub fn contains_key<Q>(&self, key: &Q) -> bool
where K: Borrow<Q> + Ord, Q: Ord + ?Sized,