pub struct VacantEntry<'a, K: Ord + Clone + Sized, V: Sized> { /* private fields */ }Available on crate feature
btree only.Expand description
Entry for a vacant key position in the tree
Implementations
Sourceimpl<'a, K: Ord + Clone + Sized, V: Sized> VacantEntry<'a, K, V>
impl<'a, K: Ord + Clone + Sized, V: Sized> VacantEntry<'a, K, V>
Sourcepub fn peak_backward(&self) -> Option<(&'a K, &'a V)>
pub fn peak_backward(&self) -> Option<(&'a K, &'a V)>
Peak previous OccupiedEntry
Sourcepub fn peak_forward(&self) -> Option<(&'a K, &'a V)>
pub fn peak_forward(&self) -> Option<(&'a K, &'a V)>
Peak the next OccupiedEntry
Sourcepub fn move_backward(self) -> Result<OccupiedEntry<'a, K, V>, Self>
pub fn move_backward(self) -> Result<OccupiedEntry<'a, K, V>, Self>
Move to previous OccupiedEntry
When reaching the front return the original entry in Err()
Sourcepub fn move_forward(self) -> Result<OccupiedEntry<'a, K, V>, Self>
pub fn move_forward(self) -> Result<OccupiedEntry<'a, K, V>, Self>
Move to next OccupiedEntry
When reaching the end, return the original entry in Err()