SegList

Struct SegList 

pub struct SegList<T> { /* private fields */ }
Segmented list with cache-friendly segment sizes

Each segment’s capacity is calculated at runtime based on T’s size to fit within a cache line.

NOTE: T is allow to larger than CACHE_LINE_SIZE, in this case SegList will ensure at least 2 items in one segment. But when T larger than 128B, you should consider put T into Box.

Implementations

Source

impl<T> SegList<T>

pub fn new() -> Self

Create a new empty SegList with one allocated segment

pub fn is_empty(&self) -> bool

Returns true if the list is empty

pub const fn segment_cap() -> usize

Get the capacity of each segment

pub fn len(&self) -> usize

Returns the total number of elements in the list

pub fn push(&mut self, item: T)

Push an element to the back of the list

pub fn pop(&mut self) -> Option<T>

Pop an element from the back of the list

pub fn iter(&self) -> SegListIter<'_, T>

Returns an iterator over the list

pub fn iter_mut(&mut self) -> SegListIterMut<'_, T>

Returns a mutable iterator over the list

pub fn drain(self) -> SegListDrain<T>

Returns a draining iterator that consumes the list and yields elements from head to tail

pub fn first(&self) -> Option<&T>

Returns a reference to the first element in the list

pub fn last(&self) -> Option<&T>

Returns a reference to the last element in the list

pub fn last_mut(&mut self) -> Option<&mut T>

Returns a mutable reference to the last element in the list

pub fn clear(&mut self)

Clear all elements from the list