pub struct SegList<T> { /* private fields */ }Expand description
Segmented list with cache-friendly segment sizes
Each segment’s capacity is calculated at runtime based on T’s size to fit within a cache line.
NOTE: T is allow to larger than CACHE_LINE_SIZE, in this case SegList will ensure at least 2
items in one segment. But when T larger than 128B, you should consider put T into Box.
Implementations
Sourceimpl<T> SegList<T>
impl<T> SegList<T>
Sourcepub const fn segment_cap() -> usize
pub const fn segment_cap() -> usize
Get the capacity of each segment
Sourcepub fn iter(&self) -> SegListIter<'_, T> ⓘ
pub fn iter(&self) -> SegListIter<'_, T> ⓘ
Returns an iterator over the list
Sourcepub fn iter_mut(&mut self) -> SegListIterMut<'_, T> ⓘ
pub fn iter_mut(&mut self) -> SegListIterMut<'_, T> ⓘ
Returns a mutable iterator over the list
Sourcepub fn drain(self) -> SegListDrain<T> ⓘ
pub fn drain(self) -> SegListDrain<T> ⓘ
Returns a draining iterator that consumes the list and yields elements from head to tail